#include "config.h"

#include "fd.h"
#include "io.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef HAVE_TIME_H
#include <time.h>
#endif
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

IO io;

int ioInit(void)
{
	io.in = STDIN_FILENO;
	io.out = STDOUT_FILENO;
	io.eof_in = 0;
	io.eof_out = 0;
	io.watermark = 0;
	io.last_read = 0;
	io.last_write = 0;
	io.total_size = 0;
	io.total_size_known = 0;
	io.total_read = 0;
	io.total_write = 0;
	io.buffer_size = 1024;
	io.buffer = 0;
	io.timeout = 250000; /* 1/4 second */
	return(0);
}

int ioBegin(void)
{
	io.buffer = (char *)malloc(sizeof(char) * io.buffer_size);
	if (io.buffer == 0) {
		fprintf(stderr, "*** ERROR: No memory\n");
		return(1);
	}
	fdBegin(io.in);
	fdBegin(io.out);
	return(0);
}

int ioEnd(void)
{
	free(io.buffer);
	io.buffer = 0;
	io.watermark = 0;
	fdEnd(io.in);
	fdEnd(io.out);
	return(0);
}

void ioCheck(void)
{
	static fd_set rset, wset;
	static struct timeval to;
	
	to.tv_sec = 0;
	to.tv_usec = io.timeout;
	io.in_ready = 1;
	io.out_ready = 1;
	FD_ZERO(&rset);
	FD_ZERO(&wset);
	if (io.watermark != io.buffer_size) FD_SET(io.in, &rset);
	if (io.watermark) FD_SET(io.out, &wset);
	if (select( ((io.in<io.out)?io.out:io.in)+1, 
		&rset, &wset, 0, &to) == -1) {
		fprintf(stderr, "*** ERROR: select() failed: %s\n", strerror(errno));
		return;
	}
	if (!FD_ISSET(io.in, &rset))
		io.in_ready = 0;
	if (!FD_ISSET(io.out, &wset))
		io.out_ready = 0;
}

int ioRead(void)
{
	int num_read = 0;

	if (!io.in_ready) return(0);
	io.last_read = 0;
	if (io.eof_in) return(0);
	if (io.watermark == io.buffer_size) return(0);
	num_read = read(io.in, 
		io.buffer + io.watermark,
		io.buffer_size - io.watermark);
	if (num_read == 0) {
		io.eof_in = 1;
		return(0);
	}
	if (num_read < 0) return(0);
	io.watermark += num_read;
	io.last_read = num_read;
	io.total_read += num_read;
	return(0);
}

int ioWrite(void)
{
	int num_written = 0;
	int count = 0;

	if (!io.out_ready) return(0);
	io.last_write = 0;
	num_written = write(io.out, io.buffer, io.watermark);
	if (num_written == 0) {
		io.eof_out = 1;
		return(0);
	}
	if (num_written < 0) return(0);
	for (count = 0; count < io.watermark - num_written; count++) {
		io.buffer[count] = io.buffer[num_written+count];
	}
	io.watermark -= num_written;
	io.last_write = num_written;
	io.total_write += num_written;
	for (count = io.watermark; count < io.buffer_size; count++) {
		io.buffer[count] = 0;
	}
	return(0);
}

int ioIsDone(void)
{
	if ((io.eof_in && (io.watermark == 0)) || io.eof_out) return(1);
	return(0);
}

